/*-
 * Copyright (c) 1991, 1993, 1994
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD: src/usr.bin/basename/basename.c,v 1.3.6.2 2001/08/02 00:59:28 obrien Exp $
 */

#include <string.h>
#include <stdlib.h>


#include "basename.h"

/***********************************************************************
 * Retourne la partie aprs le dernier slash de path
 *
 * Retourne NULL si on n'a pas pu allouer de la mmoire.
 *
 * Bas sur dirname(1) sur FreeBSD (http://www.freebsd.org/):
 * "@(#)dirname.c	8.4 (Berkeley) 5/4/95"
 **********************************************************************/
char* dirname(const char *path) {

  char *p, *garbage, *dir;

  p = (char*) strdup(path);
  if (!p)
	  return NULL;
  else
	  garbage = p;
  
  dir = malloc( (sizeof (char)) * (strlen(path) + 2));
  if (!dir) {
	  free(p);
	  return NULL;
  }
    
  memset((void*) dir, '\0', strlen(path));
  path = p;
  
  /* ceci provient presque directement de dirname(1) sur FreeBSD
   * 3.4-stable */
  
  /*
   * (1) If string is //, skip steps (2) through (5).
   * (2) If string consists entirely of slash characters, string
   *     shall be set to a single slash character.  In this case,
   *     skip steps (3) through (8).
   */
  for (p = (char*) path; *p == '/'; ++p) {
    if (!*p) {
      if (p > path)
        strcat(dir, "/");
      else
        strcat(dir, ".");
      free(garbage);
      return dir;
    }
  }
  
  /*
   * (3) If there are any trailing slash characters in string, they
   *     shall be removed.
   */
  for (; *p; ++p);
  while (*--p == '/')
    continue;
  *++p = '\0';
  
  /*
   * (4) If there are no slash characters remaining in string,
   *     string shall be set to a single period character.  In this
   *     case skip steps (5) through (8).
   *
   * (5) If there are any trailing nonslash characters in string,
   *     they shall be removed.
   */
  while (--p >= path)
    if (*p == '/')
      break;
  ++p;
  if (p == path) {
    strcat(dir, ".");
    free(garbage);
    return dir;
  }
  
  /*
   * (6) If the remaining string is //, it is implementation defined
   *     whether steps (7) and (8) are skipped or processed.
   *
   * This case has already been handled, as part of steps (1) and (2).
   */
  
  /*
   * (7) If there are any trailing slash characters in string, they
   *     shall be removed.
   */
  while (--p >= path)
    if (*p != '/')
      break;
  ++p;
  
  /*
   * (8) If the remaining string is empty, string shall be set to
   *     a single slash character.
   */
  *p = '\0';
  strcat(dir, (p == path ? "/" : path));
  free(garbage);
  return dir;
  
} /* fin dirname() */

/***********************************************************************
 * Retire la partie "fichier" d'un path
 *
 * Retourne NULL si on ne peut pas allouer de mmoire
 *
 * Donne le nom du fichier sans le path pour y accder.
 * Tir de:
 * @(#)basename.c	8.4 (Berkeley) 5/4/95
 **********************************************************************/
char* basename(const char *path) {
	
  char *p, *garbage, *base;

  p = strdup(path);
  if (!p)
	  return NULL;
  else 
	  garbage = p;

  base = malloc( (sizeof (char)) * (strlen(path) + 2));

  if (!base) {
	  free(p);
	  return NULL;
  }

  memset((void*) base, '\0', strlen(path));
  
  path = p;

  /*
   * (1) If string is // it is implementation defined whether steps (2)
   *     through (5) are skipped or processed.
   *
   * (2) If string consists entirely of slash characters, string shall
   *     be set to a single slash character.  In this case, skip steps
   *     (3) through (5).
   */
  for (p = (char*) path;; ++p) {
    if (!*p) {
      if (p > path)
        strcat(base, "/");
      else
        strcat(base, ".");
      exit(0);
    }
    if (*p != '/')
      break;
  }

  /*
   * (3) If there are any trailing slash characters in string, they
   *     shall be removed.
   */
  for (; *p; ++p)
    continue;
  while (*--p == '/')
    continue;
  *++p = '\0';
  
  /*
   * (4) If there are any slash characters remaining in string, the
   *     prefix of string up to an including the last slash character
   *     in string shall be removed.
   */
  while (--p >= path)
    if (*p == '/')
      break;
  ++p;
  
  /*
   * (5) If the suffix operand is present, is not identical to the
   *     characters remaining in string, and is identical to a suffix
   *     of the characters remaining in string, the suffix suffix
   *     shall be removed from string.
   */
  if (*path) {
    int suffixlen, stringlen, off;
    
    suffixlen = strlen(path);
    stringlen = strlen(p);
    
    if (suffixlen < stringlen) {
      off = stringlen - suffixlen;
      if (!strcmp(p + off, path))
        p[off] = '\0';
    }
  }
  strcat(base, p);
  free(garbage);
  return base;
}
